"""
Personal Kanban - Personal organizer
Copyright (C) 2017-2018 Lorenzo Ancora

This file is part of Personal Kanban.

Personal Kanban is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Personal Kanban is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

class PersonalKanban:
    """ Personal kanban with 3 virtual buckets. """
    import shelve
    from datetime import datetime

    version = 1.2

    @property
    def tasks(self):
        return self.__tasks__

    def __init__(self):
        self.__tasks__ = {}
        self.tasks["options"] = set()
        self.tasks["todo"] = set()
        self.tasks["done"] = []

        return

    def load(self, fpath: str):
        with self.shelve.open(fpath, flag='r') as db:
            self.tasks.update(db["tasks"])

        # for lab in self.tasks:
        #    val = self.tasks[lab]

        return

    def save(self, fpath: str):
        with self.shelve.open(fpath, flag='n') as db:
            db["tasks"] = self.tasks

        return

    def clear(self):
        del(self.__tasks__)
        self.__init__()

        return

    def addOption(self, name: str):
        if len(name) < 3:
            raise(ValueError("option name is too short."))
        if name not in self.tasks["options"]:
            self.tasks["options"].add(name)
        else:
            raise(NameError("option already exists."))

        return

    def removeOption(self, name: str):
        if name in self.tasks["options"]:
            self.tasks["options"].remove(name)
        else:
            raise(NameError("option does not exist."))

        return

    def promoteOption(self, name: str):
        if name not in self.tasks["options"]:
            raise(NameError("option does not exist."))
        if len(self.tasks["todo"]) >= 3:
            raise(OverflowError("no more than 3 active tasks allowed."))
        if name not in self.tasks["todo"]:
            self.tasks["todo"].add(name)
            self.removeOption(name)
        else:
            raise(NameError("todo already exists."))

        return

    def penalizeTodo(self, name: str):
        if name not in self.tasks["todo"]:
            raise(NameError("todo does not exists."))
        if name not in self.tasks["options"]:
            self.addOption(name)
            self.tasks["todo"].remove(name)
        else:
            raise(NameError("option already exists."))

        return

    def promoteTodo(self, name: str):
        if name not in self.tasks["todo"]:
            raise(NameError("todo does not exists."))
        if name not in self.tasks["done"]:
            self.tasks["done"].add((name,
                                    self.datetime.timestamp(datetime.now())))
            self.tasks["todo"].remove(name)
        else:
            self.tasks["todo"].remove(name)
            raise(Warning("task deleted (task already completed)."))

        return
